/* 
 * A dynamic calendar class 
 */

//////////////////////////////////////////////////////////////////////////////
// Constructor
function dynCalendar(thisVarName, calendarContainerId, monthSelectId, yearSelectId, editModeSelectId, editedGroupSelectId, modificationsContainerId, periodSelectId, postDataContainerId, currentDayClass, groupsTableClass) { 
  this.thisVarName = thisVarName;
    
  this.calendarContainerId = calendarContainerId;
  this.monthSelectId = monthSelectId;
  this.yearSelectId = yearSelectId;
  this.editModeSelectId = editModeSelectId;
  this.editedGroupSelectId = editedGroupSelectId;
  this.modificationsContainerId = modificationsContainerId;
  this.periodSelectId = periodSelectId;
  this.postDataContainerId = postDataContainerId;
  
  this.currentDayClass = currentDayClass;
  this.groupsTableClass = groupsTableClass;
  
  var date = new Date();
  this.currentMonth = date.getMonth();
  this.currentYear = date.getFullYear();
  this.newSpansIndex = 0;

  this.acyclicSpans = new Array();
  this.acyclicSpans[this.currentYear + '_' + this.currentMonth] = new Array();
  this.cyclicSpans = new Array();
  this.originalCyclicSpans = new Array();
  this.originalAcyclicSpans = new Array();
  this.currentAcyclicSpans = new Array();
  
  this.groups = new Array();

  this.editMode = dynCalendar.ADD_SPAN_MODE;
  this.editedGroup = 0;
  this.editSpanPeriod = 0;

  this.modifications = new Array();
  this.displayModifications = false;
}

//////////////////////////////////////////////////////////////////////////////
// Static members

dynCalendar.DAYS = new Array('lun', 'mar', 'mer', 'jeu', 'ven', 'sam', 'dim');
dynCalendar.MONTH = new Array('Janvier', 'Fvrier', 'Mars', 'Avril', 'Mai', 'Juin', 'Juillet', 'Aout', 'Septembre', 'Octobre', 'Novembre', 'Decembre');
dynCalendar.NB_DAYS_PER_WEEK = 7;
dynCalendar.NB_DAYS_PER_MONTH = 31;
dynCalendar.ADD_SPAN_MODE = 0;
dynCalendar.REMOVE_SPAN_MODE = 1;

//////////////////////////////////////////////////////////////////////////////
// Copy a two dimensional array of objects
dynCalendar.copyArray = function(type, src) {
  var dest = new Array();
  
  for (var i in src) {
    dest[i] = new type(src[i]);
  }

  return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Copy a two dimensional array of objects
dynCalendar.copyArray2 = function(type, src) {
  var dest = new Array();
  
  for (var i in src) {
    dest[i] = dynCalendar.copyArray(type, src[i]);
  }

  return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Copy a three dimensional array of objects
dynCalendar.copyArray3 = function(type, src) {
  var dest = new Array();
  
  for (var i in src) {
    dest[i] = dynCalendar.copyArray2(type, src[i]);
  }

  return dest;
}

//////////////////////////////////////////////////////////////////////////////
// Compare two dates, 
// returns -2 when d1 < d2
// returns -1 when d1 == d2 - 1 day in current month
// returns  0 when d1 == d2
// returns  1 when d1 == d2 + 1 day in current month
// returns  2 when d1 > d2
dynCalendar.compareDates = function(d1, d2) {
  var d1Year = d1.getFullYear();
  var d2Year = d2.getFullYear();
  var d1Month = d1.getMonth();
  var d2Month = d2.getMonth();
  var d1Day = d1.getDate();
  var d2Day = d2.getDate();
  
  if (d1Year > d2Year) {    
    return 2;
  } else if (d1Year == d2Year) {
    if (d1Month > d2Month) {
      return 2;
    } else if (d1Month == d2Month) {
      if (d1Day > d2Day) {
        if (d1Day == d2Day + 1) {
          return 1;
        } else {
          return 2;
        }
      } else if (d1Day == d2Day) {
        return 0;
      } else if (d1Day == d2Day - 1) {
        return -1;
      }
    }
  }

  return -2;
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.compileModifications = function() {
  var addedSpans = new Array();
  var removedSpans = new Array();  
  var modifiedSpans = new Array();  

  for (var i in this.modifications) {
    if (this.modifications[i].type == dynCalendarModification.ADD) {      
      addedSpans[-this.modifications[i].span.id] = this.modifications[i].span;
    } else if (this.modifications[i].type == dynCalendarModification.MODIFY) {
      if (this.modifications[i].span.id > 0) {
        modifiedSpans[this.modifications[i].span.id] = this.modifications[i].span;
      } else {
        addedSpans[-this.modifications[i].span.id] = this.modifications[i].span;
      }
    } else if (this.modifications[i].type == dynCalendarModification.REMOVE) {
      if (this.modifications[i].span.id > 0) {
        if (modifiedSpans[this.modifications[i].span.id]) {
          modifiedSpans[this.modifications[i].span.id] = 0;
        }
        removedSpans[this.modifications[i].span.id] = this.modifications[i].span;
      } else {
        addedSpans[-this.modifications[i].span.id] = 0;
      }
    }
  }

  return new Array(addedSpans, modifiedSpans, removedSpans);
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.setYear = function(year) {
  if (year == this.currentYear) {
    return;
  }
  
  // Copy back modifications to root spans list
  var key = this.currentYear + '_' + this.currentMonth;
  this.acyclicSpans[key] = this.currentAcyclicSpans;

  // Copy new year value to object  
  this.currentYear = year;
   
  // Copy new month spans or create a new set if none
  key = this.currentYear + '_' + this.currentMonth;
  if (!this.acyclicSpans[key]) {
    this.acyclicSpans[key] = new Array();
  }
  this.currentAcyclicSpans = dynCalendar.copyArray2(dynCalendarSpan, this.acyclicSpans[key]);
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.setMonth = function(month) {
  if (month == this.currentMonth) {
    return;
  }

  // Copy back modifications to root spans list
  var key = this.currentYear + '_' + this.currentMonth;
  this.acyclicSpans[key] = this.currentAcyclicSpans;
  
  // Treat boundary situations
  if (month == -1) {
    this.currentMonth = 11;
    this.currentYear -= 1;
  } else if (month == 12) {
    this.currentMonth = 0;    
    this.currentYear += 1;
  } else {
    this.currentMonth = month;
  }
   
  // Copy new month spans or create a new set if none
  key = this.currentYear + '_' + this.currentMonth;
  if (!this.acyclicSpans[key]) {
    this.acyclicSpans[key] = new Array();
  }
  this.currentAcyclicSpans = dynCalendar.copyArray2(dynCalendarSpan, this.acyclicSpans[key]);
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.reset = function() {
  this.modifications = new Array();
  this.originalCyclicSpans = dynCalendar.copyArray2(dynCalendarSpan, this.cyclicSpans);
  this.originalAcyclicSpans = dynCalendar.copyArray3(dynCalendarSpan, this.acyclicSpans);
  this.currentAcyclicSpans = dynCalendar.copyArray2(dynCalendarSpan, this.acyclicSpans[this.currentYear + '_' + this.currentMonth]);
  this.newSpansIndex = 0;
}

//////////////////////////////////////////////////////////////////////////////
// Put a span at end of list, spans inserted in this way should be sorted
dynCalendar.prototype.putSpan = function(span) {
  if (span.period == 0) {
    var key = span.start.getFullYear() + '_' + span.start.getMonth();
    
    if (!this.acyclicSpans[key]) {
      this.acyclicSpans[key] = new Array();
    }
    
    if (!this.acyclicSpans[key][span.group]) {
      this.acyclicSpans[key][span.group] = new Array();
    }    
    
    this.acyclicSpans[key][span.group].push(span);
  } else {
    if (!this.cyclicSpans[span.group]) {
      this.cyclicSpans[span.group] = new Array();
    }
    this.cyclicSpans[span.group].push(span);
  }
}

//////////////////////////////////////////////////////////////////////////////
// Add a span, possibly merging with existing spans
// Assumes that span is one day long (end == start)
dynCalendar.prototype.addSpan = function(span) {
  var i;
  var compareRes;
  var compareResEnd;
  var compareResPrev;
  
  if (span.period == 0) {
    if (!this.currentAcyclicSpans[this.editedGroup]) {
      this.currentAcyclicSpans[this.editedGroup] = new Array();
    }
    
    // No need to seek insertion position if this is the first inserted span
    if (!this.currentAcyclicSpans[this.editedGroup].length) {
      this.currentAcyclicSpans[this.editedGroup].push(span);
      this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
      return;
    }

    // Seek insertion point
    for (i = 0 ; i < this.currentAcyclicSpans[this.editedGroup].length ; i++) {
      compareRes = dynCalendar.compareDates(span.start, this.currentAcyclicSpans[this.editedGroup][i].start);
      compareResEnd = dynCalendar.compareDates(span.end, this.currentAcyclicSpans[this.editedGroup][i].end);
      
      if (compareRes == -2) {
        if (i == 0) {
          // Simple add case
          this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
        } else {
          // See if previous span can be merged with inserted span
          compareResPrev = dynCalendar.compareDates(span.start, this.currentAcyclicSpans[this.editedGroup][i - 1].end);
          
          if (compareResPrev == 1) {
            // Add after case
            // Merge with previous span if comments are equal, or add a new span
            if (span.comments == this.currentAcyclicSpans[this.editedGroup][i - 1].comments) {
              this.currentAcyclicSpans[this.editedGroup][i - 1].end = span.start;
              this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i - 1]));
            } else {
              this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
              this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
            }
          } else if (compareResPrev == 2) {
            // Simple add case
            this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
            this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
          } else if (compareResPrev == 0) {
            // Add at end case
            if (span.comments != this.currentAcyclicSpans[this.editedGroup][i - 1].comments) {
              // Add a new span and move previous day one day earlier
              this.currentAcyclicSpans[this.editedGroup][i - 1].end.setDate(this.currentAcyclicSpans[this.editedGroup][i - 1].end.getDate() - 1);
              this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i - 1]));
              
              this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
              this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
            }
          }
        }
        
        break;
      } else if (compareRes == -1) {
        if (i == 0) {
          // Add before case
          if (span.comments == this.currentAcyclicSpans[this.editedGroup][i].comments) {
            this.currentAcyclicSpans[this.editedGroup][i].start = span.start;
            this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i]));
          } else {
            this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
            this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
          }
        } else {
          compareResPrev = dynCalendar.compareDates(span.start, this.currentAcyclicSpans[this.editedGroup][i - 1].end);
          if (compareResPrev == 1) {
            // Join case
            if (this.currentAcyclicSpans[this.editedGroup][i - 1].comments == span.comments) {
              if (span.comments == this.currentAcyclicSpans[this.editedGroup][i].comments) {
                this.currentAcyclicSpans[this.editedGroup][i - 1].end = this.currentAcyclicSpans[this.editedGroup][i].end;
                this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i - 1]));
                
                this.modifications.push(new dynCalendarModification(dynCalendarModification.REMOVE, this.currentAcyclicSpans[this.editedGroup][i]));
                this.currentAcyclicSpans[this.editedGroup].splice(i, 1);
              } else {
                this.currentAcyclicSpans[this.editedGroup][i - 1].end = span.end;
                this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i - 1]));
              }
            } else {
              if (span.comments == this.currentAcyclicSpans[this.editedGroup][i].comments) {
                this.currentAcyclicSpans[this.editedGroup][i].start = span.start;
                this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i]));
              } else {
                this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
                this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
              }
            }
          } else if (compareResPrev == 2) {
            // Add before case
            if (span.comments == this.currentAcyclicSpans[this.editedGroup][i].comments) {
              this.currentAcyclicSpans[this.editedGroup][i].start = span.start;
              this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i]));
            } else {
              this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
              this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
            }
          }
        }
        
        break;
      } else if (compareRes == 0 && compareResEnd == 0) {
        // Replace case
        if (span.comments != this.currentAcyclicSpans[this.editedGroup][i].comments) {
          this.currentAcyclicSpans[this.editedGroup][i].comments = span.comments;
          this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i]));
        }          
      } else if (compareRes == 0) {
        // Add at begining case
        if (span.comments != this.currentAcyclicSpans[this.editedGroup][i].comments) {
          // Add a new span and move current span start one day after
          this.currentAcyclicSpans[this.editedGroup][i].start.setDate(this.currentAcyclicSpans[this.editedGroup][i].start.getDate() + 1);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i]));
          
          this.currentAcyclicSpans[this.editedGroup].splice(i, 0, span);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
        }

        break;
      } else if ((compareRes == 2 && compareResEnd == -2) ||
                 (compareRes == 2 && compareResEnd == -1) || 
                 (compareRes == 1 && compareResEnd == -2) ||
                 (compareRes == 1 && compareResEnd == -1)) {
        // Add in the middle case
        if (span.comments != this.currentAcyclicSpans[this.editedGroup][i].comments) {
          var newSpan = new dynCalendarSpan(span.group, span.end, this.currentAcyclicSpans[this.editedGroup][i].end, 0, this.currentAcyclicSpans[this.editedGroup][i].comments);
          newSpan.start.setDate(span.end.getDate() + 1);
          
          this.currentAcyclicSpans[this.editedGroup][i].end.setDate(span.start.getDate() - 1);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i]));
          
          this.currentAcyclicSpans[this.editedGroup].splice(i + 1, 0, span, newSpan);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
          this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, newSpan));
        }

        break;
      } else if ((compareRes == 1 && compareResEnd == 0) ||
                 (compareRes == 2 && compareResEnd == 0)) {
        // Add at end case
        if (span.comments != this.currentAcyclicSpans[this.editedGroup][i].comments) {
          this.currentAcyclicSpans[this.editedGroup][i].end.setDate(span.start.getDate() - 1);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i]));
          
          this.currentAcyclicSpans[this.editedGroup].splice(i + 1, 0, span);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
        }

        break;
      }
    }

    if (i == this.currentAcyclicSpans[this.editedGroup].length) {
      compareRes = dynCalendar.compareDates(span.start, this.currentAcyclicSpans[this.editedGroup][i - 1].end);
      if (compareRes == 1) {
        // Add after case
        if (span.comments == this.currentAcyclicSpans[this.editedGroup][i - 1].comments) {
          this.currentAcyclicSpans[this.editedGroup][i - 1].end = span.start;
          this.modifications.push(new dynCalendarModification(dynCalendarModification.MODIFY, this.currentAcyclicSpans[this.editedGroup][i - 1]));
        } else {
          this.currentAcyclicSpans[this.editedGroup].push(span);
          this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
        }        
      } else if (compareRes == 2) {
        // Simple add case
        this.currentAcyclicSpans[this.editedGroup].push(span);
        this.modifications.push(new dynCalendarModification(dynCalendarModification.ADD, span));
      }
    }
  } else {
    // Simply put cyclic spans at end of list
    if (!this.cyclicSpans[span.group]) {
      this.cyclicSpans[span.group] = new Array();
    }
    this.cyclicSpans[span.group].push(span);
  }
}

//////////////////////////////////////////////////////////////////////////////
// Remove a span
dynCalendar.prototype.subSpan = function(date) {
  // Search hit span
  if (this.editSpanPeriod == 0) {
    for (var i in this.currentAcyclicSpans[this.editedGroup]) {
      if (dynCalendar.compareDates(this.currentAcyclicSpans[this.editedGroup][i].start, date) == 0) {
        this.modifications.push(new dynCalendarModification(dynCalendarModification.REMOVE, this.currentAcyclicSpans[this.editedGroup][i]));
        this.currentAcyclicSpans[this.editedGroup].splice(i, 1);
      }
    }
  } else {
    for (var i in this.cyclicSpans[this.editedGroup]) {
      if (dynCalendar.compareDates(this.cyclicSpans[this.editedGroup][i].start, date) == 0) {
        this.modifications.push(new dynCalendarModification(dynCalendarModification.REMOVE, this.cyclicSpans[this.editedGroup][i]));
        this.cyclicSpans[this.editedGroup].splice(i, 1);
      }
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
// Add a group, returns it's number (starting from 0)
dynCalendar.prototype.addGroup = function(group) {
  this.groups.push(group);
  return this.groups.length - 1;
}

//////////////////////////////////////////////////////////////////////////////
// Render a table containing colored bands for each group
dynCalendar.prototype.renderGroups = function(groups) {
  var i;
  var j;
  var out;

  out = '<table class="' + this.groupsTableClass + '">';

  j = groups.length - 1;
  for (i = this.groups.length - 1 ; i >= 0 ; i--) {
    if (this.inArray(groups, i)) {
      out += '<tr><td style="background: ' + this.groups[groups[j]].color + ';"></td></tr>';
      j--;
    } else {
      out += '<tr><td style="background: none;"></td></tr>';
    }
  }

  out += '</table>';

  return out;
}

//////////////////////////////////////////////////////////////////////////////
// Returns true if the given array contains value
dynCalendar.prototype.inArray = function(array, value) {
  var i;
  
  for (i = 0 ; i < array.length ; i++) {
    if (array[i] == value) {
      return true;
    }
  }
  
  return false;
}

//////////////////////////////////////////////////////////////////////////////
// Returns an array containing the spans groups that owns given day
dynCalendar.prototype.retreiveSpanGroups = function(date) {
  var res = new Array();
  var i;
  var j;
  
  // Search acyclic spans  
  for (j in this.currentAcyclicSpans) {
    for (i = 0 ; i < this.currentAcyclicSpans[j].length ; i++) {
      if (this.currentAcyclicSpans[j][i].contains(date)) {
        if (!this.inArray(res, this.currentAcyclicSpans[j][i].group)) {
          res.push(this.currentAcyclicSpans[j][i].group);
        }
      }
    }
  }

  // Sort results
  res.sort();

  return res;
}

//////////////////////////////////////////////////////////////////////////////
// Convert a week day id from builtin Date() format (0 = sunday) to our format (0 = monday)
dynCalendar.prototype.getWeekDay = function(day) {
  return (day + dynCalendar.NB_DAYS_PER_WEEK - 1) % dynCalendar.NB_DAYS_PER_WEEK;
}

//////////////////////////////////////////////////////////////////////////////
// Render the calendar at the given month and year to calendarContainerId element
dynCalendar.prototype.render = function() {
  // Initialise variables
  var date = new Date();
  var out = '';
  var i;

  // Save currentDate elements
  var currentYear = date.getFullYear();
  var currentMonth = date.getMonth();
  var currentDay = date.getDate();

  // Set date object to the first day of desired month
  date.setDate(1);
  date.setMonth(this.currentMonth);
  date.setFullYear(this.currentYear);

  // Render calendar
  out = '<table><tr>';

  for (i = 0 ; i < dynCalendar.NB_DAYS_PER_WEEK ; i++) {
    out += '<th>' + dynCalendar.DAYS[i] + '</th>';
  }

  out += '</tr><tr>';

  // Fill blanks before month's first week day
  for (i = 0 ; i < this.getWeekDay(date.getDay()) ; i++) {
    out += '<td></td>';
  }

  for (i = 0 ; i < dynCalendar.NB_DAYS_PER_MONTH ; i++) {
    var day = date.getDate();
    var groups = this.retreiveSpanGroups(date);
    var weekDay = this.getWeekDay(date.getDay());
    var groupsStr = '';
    
    if (day > i) {
      if (weekDay == 0) {
        // Monday, start row
        out += '<tr>';
      }      

      var dayId = 'day_' + this.currentYear + '_' + this.currentMonth + '_' + day;
      out += '<td id="' + dayId + '" onclick="' + this.thisVarName + '.onClickDay(new Date(' + this.currentYear + ', ' + this.currentMonth + ', ' + day + '))"';

      if (currentDay == day && currentMonth == this.currentMonth && currentYear == this.currentYear) {
        out += ' class="' + this.currentDayClass + '"><div><span>' + day + '</span>' + this.renderGroups(groups) + '</div></td>';
      } else {
        out += '><div><span>' + day + '</span>' + this.renderGroups(groups) + '</div></td>';
      }

      if (weekDay == dynCalendar.NB_DAYS_PER_WEEK - 1) {
        // Reached a sunday, close row
        out += '</tr>';
      }
    } else {
      break;
    }

    date.setDate(date.getDate() + 1);
  }

  // Eventually close row, and close table
  if (weekDay != dynCalendar.NB_DAYS_PER_WEEK - 1) {
    out += '</tr>';
  }
  out += '</table>';

  // Output calendar to page
  var outElement = document.getElementById(this.calendarContainerId);
  outElement.innerHTML = out;

  // Update month select
  outElement = document.getElementById(this.monthSelectId);
  outElement.value = this.currentMonth;

  // Update year select
  outElement = document.getElementById(this.yearSelectId);
  outElement.value = this.currentYear;

  // Update other parts
  this.renderModifications();
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.renderModifications = function() {
  var out = '';
  var i;
  var compiledModifications = this.compileModifications();

  if (this.displayModifications) {
    for (i in compiledModifications[0]) {
      if (compiledModifications[0][i] != 0) {
        out += 'ADD ' + compiledModifications[0][i].id + '<br />';
      }
    }
  
    for (i in compiledModifications[1]) {
      if (compiledModifications[1][i] != 0) {
        out += 'MODIFY ' + compiledModifications[1][i].id + '<br />';
      }
    }
  
    for (i in compiledModifications[2]) {
      out += 'REMOVE ' + compiledModifications[2][i].id + '<br />';
    }

    out += '<hr />';
  
    for (i = 0 ; i < this.modifications.length ; i++) {
      if (this.modifications[i].type == dynCalendarModification.ADD) {
        out += 'ADD ' + this.modifications[i].span.id + '<br />';
      } else if (this.modifications[i].type == dynCalendarModification.REMOVE) {
        out += 'REMOVE ' + this.modifications[i].span.id + '<br />';
      } else if (this.modifications[i].type == dynCalendarModification.MODIFY) {
        out += 'MODIFY ' + this.modifications[i].span.id + '<br />';
      }
    }
  }

  // Output to page
  var outElement = document.getElementById(this.modificationsContainerId);
  outElement.innerHTML = out;
}

//////////////////////////////////////////////////////////////////////////////
// Event handlers
dynCalendar.prototype.onPreviousMonth = function() {
  this.setMonth(this.currentMonth - 1);

  // Update display
  this.render();
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onNextMonth = function() {
  this.setMonth(this.currentMonth + 1);

  // Update display
  this.render();
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onSelChangeYear = function() {
  var inElement = document.getElementById(this.yearSelectId);  
  this.setYear(inElement.value * 1);

  // Update display
  this.render();
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onSelChangeMonth = function() {
  var inElement = document.getElementById(this.monthSelectId);  
  this.setMonth(inElement.value * 1);  

  // Update display
  this.render();
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onChangeEditMode = function() {
  var inElement = document.getElementById(this.editModeSelectId);
  this.editMode = inElement.value * 1;
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onChangeEditedGroup = function() {
  var inElement = document.getElementById(this.editedGroupSelectId);
  this.editedGroup = inElement.value * 1;
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onClickDay = function(date) {
  //alert(date.getDate() + '/' + date.getMonth() + '/' + date.getFullYear());

  if (this.editMode == dynCalendar.ADD_SPAN_MODE) {
    this.addSpan(new dynCalendarSpan(this.newSpansIndex, this.editedGroup, date, date, this.editSpanPeriod, ''));
    this.newSpansIndex -= 1;

    // Update display
    var dayId = 'day_' + date.getFullYear() + '_' + date.getMonth() + '_' + date.getDate();
    var outElement = document.getElementById(dayId);
    var groups = this.retreiveSpanGroups(date);
    outElement.innerHTML = '<div><span>' + date.getDate() + '</span>' + this.renderGroups(groups) + '</div>';
    
    this.renderModifications();    
  } else if (this.editMode == dynCalendar.REMOVE_SPAN_MODE) {
    this.subSpan(date);

    // Update display
    this.render();
  }
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onDisplayModifications = function() {
  this.displayModifications = !this.displayModifications;
  this.renderModifications();
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onCancelModifications = function() {
  this.cyclicSpans = dynCalendar.copyArray2(dynCalendarSpan, this.originalCyclicSpans);
  this.acyclicSpans = dynCalendar.copyArray3(dynCalendarSpan, this.originalAcyclicSpans);  
  this.modifications = new Array();
  
  // Copy current month spans or create a new set if none
  var key = this.currentYear + '_' + this.currentMonth;
  if (!this.acyclicSpans[key]) {
    this.acyclicSpans[key] = new Array();
  }
  this.currentAcyclicSpans = dynCalendar.copyArray2(dynCalendarSpan, this.acyclicSpans[key]);

  // Update display
  this.render();
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onChangePeriod = function() {
  var inElement = document.getElementById(this.periodSelectId);
  this.editSpanPeriod = inElement.value * 1;
}

//////////////////////////////////////////////////////////////////////////////
dynCalendar.prototype.onSubmit = function() {
  var outElement = document.getElementById(this.postDataContainerId);
  var compiledModifications = this.compileModifications();
  var out = '';
  var i, j;

  // Copy modifications to form
  
  j = 0;
  for (i in compiledModifications[0]) {
    if (compiledModifications[0][i] != 0) {
      out += '<input type="hidden" name="addedSpans[' + j + ']" value="' + compiledModifications[0][i] + '" />';
      j++;
    }
  }

  j = 0;
  for (i in compiledModifications[1]) {
    if (compiledModifications[1][i] != 0) {
      out += '<input type="hidden" name="modifiedSpans[' + j + ']" value="' + compiledModifications[1][i] + '" />';
      j++;
    }
  }

  j = 0;
  for (i in compiledModifications[2]) {
    out += '<input type="hidden" name="removedSpans[' + j + ']" value="' + compiledModifications[2][i].id + '" />';
    j++;
  }

  outElement.innerHTML = out;
    
  return true;
}
